@@ -4,7 +4,9 @@ from __future__ import unicode_literals, print_function
 import getpass
 import os
 import re
+import sys
 import shutil
+from binascii import hexlify
 try:
     import ConfigParser
 except ImportError:
@@ -14,10 +16,14 @@ from rarfile import RarFile
 from glob import iglob
 
 from pyasn1.codec.der import decoder
-from cprocsp import csp
+from cprocsp import csp, cryptoapi
 
 from . import KeysetException
 
+if sys.version_info >= (3,):
+        ord = lambda x: x
+        unicode = lambda x: str(x, sys.getdefaultencoding())
+
 
 def _rarsplit(fn):
     res = re.split(r'[\\/]', fn, maxsplit=1)
@@ -33,6 +39,19 @@ def _rarbase(fn):
     return fn
 
 
+def _cert_info(certdata):
+    # return cryptoapi.cert_info(certdata)
+    tc = csp.Cert(certdata)
+    ci = csp.CertInfo(tc)
+    return dict(
+        name=unicode(ci.name()),
+        thumb=unicode(hexlify(tc.thumbprint())),
+        issuer=unicode(ci.issuer()),
+        serial=':'.join(hex(ord(x))[2:] for x in
+                        reversed(ci.serial())),
+    )
+
+
 def _ensure(flag, msg, exc=KeysetException):
     if not flag:
         raise exc(msg)
@@ -230,3 +249,36 @@ class hdimage(object):
             key = res.ctx.get_key(keyspec)
             key.store_cert(newc)
         return res
+
+    @classmethod
+    def query(cls, filename, filetype=None):
+        """Determine container name from archive contents
+
+        :filename: archive file name
+        :filetype: archive type ('rar', 'zip'). Default: autodetect
+        :returns: container info in dictionary
+
+        """
+        res = {}
+        ft = filetype or filename.lower()
+        if ft.endswith('rar'):
+            arccls = RarFile
+        elif ft.endswith('zip'):
+            arccls = ZipFile
+        else:
+            _ensure(False, 'Unsupported file type: {0}'.format(ft))
+
+        zf = arccls(filename)
+        names = zf.namelist()
+        struct = cls.check_structure(names)
+        value = zf.read('{0}/name.key'.format(struct['dirname']))
+        res['name'] = decoder.decode(value)[0][0]
+        res['certs'] = {}
+
+        for nsc in struct['non_standard']:
+            certdata = zf.read(nsc)
+            res['certs']['AT_KEYEXCHANGE'] = _cert_info(certdata)
+        for nsc in struct['standard']:
+            certdata = zf.read(nsc + '.cer')
+            res['certs'][nsc] = _cert_info(certdata)
+        return res
